@@ -18,7 +18,12 @@
 # See http://www.gnu.org/licenses/ for more information.
 
 """
-The items a music expression is constructed with in a tree structure.
+The Reader is used to construct a tree structure of (musical and other)
+items in a LilyPond document. The item types that are used are in the items
+module.
+
+You instantiate a Reader with a ly.document.Source that reads tokens from
+any (part of a) ly.document.Document.
 
 Whitespace and comments are left out.
 
@@ -138,9 +143,12 @@ class Reader(object):
             return True
 
     def add_duration(self, item, token=None, source=None):
-        """Add a duration attribute to the item."""
+        """Add a duration attribute to the item.
+        
+        When there are tokens, a Duration item is also appended to the item.
+        
+        """
         source = source or self.source
-        d = item.duration = self.factory(Duration, position=0)
         tokens = []
         if not token or isinstance(token, lilypond.Duration):
             if token:
@@ -155,11 +163,12 @@ class Reader(object):
                     self.source.pushback()
                     break
         if tokens:
-            d.tokens = tuple(tokens)
-            d.position = tokens[0].pos
-            d.base_scaling = self.prev_duration = ly.duration.base_scaling(tokens)
+            d = self.factory(Duration, tokens[0])
+            d.tokens = tuple(tokens[1:])
+            item.append(d)
+            item.duration = self.prev_duration = ly.duration.base_scaling(tokens)
         else:
-            d.base_scaling = self.prev_duration
+            item.duration = self.prev_duration
     
     def consume(self, last_token=None):
         """Yield the tokens from our source until a parser is exit.
@@ -301,8 +310,7 @@ class Reader(object):
     @_tokencls(lilypond.ChordSeparator)
     def read_chord_specifier(self, t, source=None):
         """Read stuff behind notes in chordmode."""
-        item = self.factory(ChordSpecifier)
-        item.position = t.pos
+        item = self.factory(ChordSpecifier, position=t.pos)
         item.append(self.factory(ChordItem, t))
         for t in self.consume():
             if isinstance(t, lilypond.ChordItem):
@@ -321,17 +329,16 @@ class Reader(object):
         item = self.factory(Tremolo, t)
         for t in self.source:
             if isinstance(t, lilypond.TremoloDuration):
-                item.duration = self.factory(Duration, t)
-                item.duration.base_scaling = ly.duration.base_scaling_string(t)
+                item.append(self.factory(Duration, t))
+                item.duration = ly.duration.base_scaling_string(t)
             else:
                 self.source.pushback()
             break
         return item
     
-    @_tokencls(lilypond.Name)
+    @_tokencls(lilypond.Name, lilypond.ContextProperty)
     def handle_name(self, t, source):
-        if self.source.state.depth() < 2:
-            return self.read_assignment(t)
+        return self.read_assignment(t)
     
     @_tokencls(
         lilypond.PaperVariable,
@@ -558,14 +565,25 @@ class Reader(object):
         if t == '\\scaleDurations':
             for i in self.read(source):
                 item.append(i)
-                if isinstance(i, Scheme):
-                    pair = i.get_pair_ints()
-                    if pair:
-                        item.scaling = Fraction(*pair)
+                if isinstance(i, Number):
+                    item.scaling = i.value()
+                elif isinstance(i, Scheme):
+                    try:
+                        pair = i.get_pair_ints()
+                        if pair:
+                            item.scaling = Fraction(*pair)
+                        else:
+                            val = i.get_fraction()
+                            if val is not None:
+                                item.scaling = val
+                    except ZeroDivisionError:
+                        pass
                 break
         elif t == '\\tuplet':
             for t in source:
                 if isinstance(t, lilypond.Fraction):
+                    item.append(self.factory(Number, t))
+                    item.numerator, item.denominator = map(int, t.split('/'))
                     item.scaling = 1 / Fraction(t)
                 elif isinstance(t, lilypond.Duration):
                     self.add_duration(item, t, source)
@@ -576,6 +594,8 @@ class Reader(object):
         else: # t == '\\times'
             for t in source:
                 if isinstance(t, lilypond.Fraction):
+                    item.append(self.factory(Number, t))
+                    item.numerator, item.denominator = map(int, t.split('/'))
                     item.scaling = Fraction(t)
                     break
                 elif not isinstance(t, lex.Space):
@@ -660,27 +680,32 @@ class Reader(object):
     @_commands('\\tempo')
     def handle_tempo(self, t, source):
         item = self.factory(Tempo, t)
-        item._text = None
-        item._tempo = []
         source = self.consume()
         equal_sign_seen = False
+        text_seen = False
+        t = None
         for t in source:
             if not equal_sign_seen:
-                if not item._text:
-                    if isinstance(t, lilypond.SchemeStart):
-                        item._text = self.read_scheme_item(t)
-                    elif isinstance(t, lex.StringStart):
-                        item._text = self.factory(String, t, True)
-                    elif isinstance(t, lilypond.Markup):
-                        item._text = self.handle_markup(t)
+                if (not text_seen and isinstance(t,
+                        (lilypond.SchemeStart, lex.StringStart, lilypond.Markup))):
+                    item.append(self.read_item(t))
+                    t = None
+                    text_seen = True
                 elif isinstance(t, lilypond.Length):
                     self.add_duration(item, t, source)
+                    t = None
                 elif isinstance(t, lilypond.EqualSign):
+                    item.tokens = (t,)
                     equal_sign_seen = True
-            elif isinstance(t, lilypond.IntegerValue):
-                item._tempo.append(t)
-            elif isinstance(t, lilypond.SchemeStart):
-                item._tempo.append(self.read_scheme_item(t))
+                    t = None
+            elif isinstance(t, (lilypond.IntegerValue, lilypond.SchemeStart)):
+                item.append(self.read_item(t))
+            elif t == "-":
+                item.tokens += (t,)
+        ## if the last token does not belong to the \\tempo expression anymore,
+        ## push it back
+        if t and not isinstance(t, (lex.Space, lex.Comment)):
+            self.source.pushback()
         return item
     
     @_commands('\\time')
@@ -778,7 +803,8 @@ class Reader(object):
                 break
         for t in skip(self.consume()):
             i = self.read_lyric_item(t) or self.read_item(t)
-            item.append(i)
+            if i:
+                item.append(i)
             break
         return item
     
@@ -986,6 +1012,11 @@ class Reader(object):
         self.add_markup_arguments(item)
         return item
     
+    @_markup(lilypond.MarkupUserCommand)
+    def handle_markup_user_command(self, t):
+        item = self.factory(MarkupUserCommand, t)
+        return item
+    
     @_markup(lilypond.OpenBracketMarkup)
     def handle_markup_open_bracket(self, t):
         item = self.factory(MarkupList, t)
